/* -*-C-*- dos2ux.c */
/*-->dos2ux*/
/**********************************************************************/
/****************************** dos2ux ********************************/
/**********************************************************************/

/***********************************************************************
Convert one or more files from DOS to Unix format (\r\n -> \n),
preserving time stamps.

Usage:
	dos2ux filelist

[19-Jul-89]
***********************************************************************/

#include <stdio.h>
#include <errno.h>
extern int errno;

#include "os.h"
#include "stat.h"
#include "stdlib.h"
#include "types.h"
#include "unixlib.h"

#if    IBM_PC_TURBO
#include <dos.h>	/* for struct date and struct time */
#include <io.h>		/* for struct ftime */
#endif /* IBM_PC_TURBO */

#include <time.h>	/* for time_t */

#if    KCC_20
#undef WB_OPEN
#define WB_OPEN		"wb7"
#endif /* KCC_20 */

#define TMPFILE "dos2ux.$$$"

int	dos2ux ARGS((char  *file));
int	main ARGS((int argc,char  *argv[]));
int	utime ARGS((const char  *path,time_t times[2]));

int
main(argc,argv)
int argc;
char  *argv[];
{
    int k;
    int errors = 0;

    for (k = 1; k < argc; ++k)
	errors += dos2ux(argv[k]);
    return (errors);
}

int
dos2ux(file)
char  *file;
{
    FILE  *fpin;
    FILE  *fpout;
    int c;
    struct stat inbuf;
    time_t times[2];

    (void)fprintf(stderr,"%s",file);

    if ((fpin = fopen(file,RB_OPEN)) == (FILE*)NULL)
    {
	(void)fprintf(stderr," - [FAILED]\n");
	return (1);
    }
    if ((fpout = fopen(TMPFILE,WB_OPEN)) == (FILE*)NULL)
    {
	(void)fprintf(stderr," [%s] - [FAILED]\n",TMPFILE);
	return (1);
    }

    while ((c = getc(fpin)) != EOF)
    {
	if (c == '\r')			/* peek ahead to see if we have \r\n */
	{
	    c = getc(fpin);
	    if (c != '\n')
	    {
		(void)ungetc(c,fpin);
		c = '\r';
	    }
	}
	putc(c,fpout);
    }
    if (fstat(fileno(fpin),&inbuf) == -1)
    {
	(void)fprintf(stderr," [%s] - [FAILED]\n",TMPFILE);
	return (1);
    }

    times[0] = inbuf.st_atime;
    times[1] = inbuf.st_mtime;
    (void)fclose(fpin);
    (void)fclose(fpout);
    unlink(file);
    rename(TMPFILE,file);
    if (utime(file,times) == 0)
    {
	(void)fprintf(stderr," [OK]\n");
	return (0);
    }
    else
    {
	(void)fprintf(stderr," [FAILED]\n");
	return (1);
    }
}

#if    IBM_PC_TURBO
int
utime(path,times)		/* set file access/modification times */
const char	*path;			/* full filename */
time_t		times[2];		/* access and modification times */
{
	int		code;
	struct date	dt;
	FILE		*fp;
	struct ftime	ft;
	struct time	tt;

	fp = fopen(path,"r");
	if (fp == (FILE*)NULL)
	{
		errno = ENOENT;	/* fopen() should set this, but doesn't */
		return (-1);
	}

	if (times == (time_t*)NULL)	/* use current time */
		(void)unixtodos(time((time_t*)NULL), &dt, &tt);
	else	/* use modification time (DOS has no access time) */
		(void)unixtodos(times[1], &dt, &tt);

	ft.ft_tsec  = (unsigned)tt.ti_sec / 2; /* ft_tsec: 2-sec intervals */
	ft.ft_min   = (unsigned)tt.ti_min;
	ft.ft_hour  = (unsigned)tt.ti_hour;
	ft.ft_day   = (unsigned)dt.da_day;
	ft.ft_month = (unsigned)dt.da_mon;
	ft.ft_year  = ((unsigned)dt.da_year) - 1980;

	code = setftime(fileno(fp),&ft);
	(void)fclose(fp);

	return (code);
}
#endif /* IBM_PC_TURBO */
